@@ -27,155 +27,180 @@ my $logger = Log::Log4perl::get_logger('main');
 =head2 new
 
     Initialize a Biber::DataModel object
+    We are passing in an array of datamodels as there may be more than one in tool
+    mode - the one from biber-tool.conf and modifications in a user .conf
+    Later takes precedence.
 
 =cut
 
 sub new {
   my $class = shift;
-  my $dm = shift;
+  my $dms = shift;
   my $self;
   $self = bless {}, $class;
-#  use Data::Dump;dd($dm);exit 0;
+  # use Data::Dump;dd($dms);exit 0;
 
-  # Early check for fatal datamodel errors
+  foreach my $dm ($dms->@*) { # Could potentially be more than one data model in future
 
-  # Make sure dates are named *date. A lot of code relies on this.
-  foreach my $date (grep {$_->{datatype} eq 'date'} $dm->{fields}{field}->@*) {
-    unless ($date->{content} =~ m/date$/) {
-      biber_error("Fatal datamodel error: date field '" . $date->{content} . "' must end with string 'date'");
+    # First, we normalise all entrytypes and fields to case-folded form for internal
+    # comparisons but we save a map of case-folded variants to actual names
+    # so that we can recover the information later for output
+    foreach my $et ($dm->{entrytypes}{entrytype}->@*) {
+      $self->{casemap}{foldtoorig}{fc($et->{content})} = $et->{content};
+      $et->{content} = fc($et->{content});
+    }
+    foreach my $f ($dm->{fields}{field}->@*) {
+      $self->{casemap}{foldtoorig}{fc($f->{content})} = $f->{content};
+      $f->{content} = fc($f->{content});
     }
-  }
 
-  # Pull out legal entrytypes, fields and constraints and make lookup hash
-  # for quick tests later
-  foreach my $f ($dm->{fields}{field}->@*) {
+    # Early check for fatal datamodel errors
+    # Make sure dates are named *date. A lot of code relies on this.
+    foreach my $date (grep {$_->{datatype} eq 'date'} $dm->{fields}{field}->@*) {
+      unless ($date->{content} =~ m/date$/) {
+        biber_error("Fatal datamodel error: date field '" . $date->{content} . "' must end with string 'date'");
+      }
+    }
 
-    # In case of conflicts, we need to remove the previous definitions since
-    # later overrides earlier
-    if (my $previous = $self->{fieldsbyname}{$f->{content}}) {
+    # Multiscript enabled fields
+    foreach my $f ($dm->{multiscriptfields}{field}->@*) {
+      $self->{multiscriptfields}{$f} = 1;
+    }
 
+    # Pull out legal entrytypes, fields and constraints and make lookup hash
+    # for quick tests later
+    foreach my $f ($dm->{fields}{field}->@*) {
+
+      # In case of conflicts, we need to remove the previous definitions since
+      # later overrides earlier
+      if (my $previous = $self->{fieldsbyname}{$f->{content}}) {
+
+        if ($f->{format}) {
+          $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@*;
+        }
+        $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@*;
+        $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@*;
+        $self->{fieldsbydatatype}{$previous->{datatype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbydatatype}{$previous->{datatype}}->@*;
+        $self->{fieldsbyformat}{$previous->{'format'}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyformat}{$previous->{format}}->@*;
+        delete $self->{fieldsbyname}{$f->{content}};
+      }
+
+      $self->{fieldsbyname}{$f->{content}} = {'fieldtype'   => $f->{fieldtype},
+                                              'datatype'    => $f->{datatype},
+                                              'format'      => $f->{format} || 'default'};
       if ($f->{format}) {
-        $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{$previous->{format}}->@*;
+        push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{$f->{format}}->@*, $f->{content};
+      }
+      push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{'*'}->@*, $f->{content};
+      push $self->{fieldsbyfieldtype}{$f->{fieldtype}}->@*, $f->{content};
+      push $self->{fieldsbydatatype}{$f->{datatype}}->@*, $f->{content};
+      push $self->{fieldsbyformat}{$f->{format} || 'default'}->@*, $f->{content};
+
+      # check null_ok
+      if ($f->{nullok}) {
+        $self->{fieldsbyname}{$f->{content}}{nullok} = 1;
+      }
+      # check skips - fields we don't want to output to .bbl
+      if ($f->{skip_output}) {
+        $self->{fieldsbyname}{$f->{content}}{skipout} = 1;
       }
-      $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@* = grep {$_ ne $f->{content}} $self->{fieldsbytype}{$previous->{fieldtype}}{$previous->{datatype}}{'*'}->@*;
-      $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyfieldtype}{$previous->{fieldtype}}->@*;
-      $self->{fieldsbydatatype}{$previous->{datatype}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbydatatype}{$previous->{datatype}}->@*;
-      $self->{fieldsbyformat}{$previous->{'format'}}->@* = grep {$_ ne $f->{content}} $self->{fieldsbyformat}{$previous->{format}}->@*;
-      delete $self->{fieldsbyname}{$f->{content}};
     }
 
-    $self->{fieldsbyname}{$f->{content}} = {'fieldtype'   => $f->{fieldtype},
-                                            'datatype'    => $f->{datatype},
-                                            'format'      => $f->{format} || 'default'};
-    if ($f->{format}) {
-      push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{$f->{format}}->@*, $f->{content};
+    my $constants;
+    foreach my $constant ($dm->{constants}{constant}->@*) {
+      $self->{constants}{$constant->{name}}{type} = $constant->{type};
+      $self->{constants}{$constant->{name}}{value} = $constant->{content};
     }
-    push $self->{fieldsbytype}{$f->{fieldtype}}{$f->{datatype}}{'*'}->@*, $f->{content};
-    push $self->{fieldsbyfieldtype}{$f->{fieldtype}}->@*, $f->{content};
-    push $self->{fieldsbydatatype}{$f->{datatype}}->@*, $f->{content};
-    push $self->{fieldsbyformat}{$f->{format} || 'default'}->@*, $f->{content};
 
-    # check null_ok
-    if ($f->{nullok}) {
-      $self->{fieldsbyname}{$f->{content}}{nullok} = 1;
-    }
-    # check skips - fields we don't want to output to .bbl
-    if ($f->{skip_output}) {
-      $self->{fieldsbyname}{$f->{content}}{skipout} = 1;
-    }
-  }
+    # Get entrytpes from existing model (meaning, already processed XML model)
+    # or XML model. We need to do this in case we have two datamodel sources like
+    # a default tool mode config and a user config
+    foreach my $et (($dm->{entrytypes}{entrytype}->@*, keys $self->{entrytypesbyname}->%*)) {
+      my $es;
+      if (ref($et) eq 'HASH') { # from new config
+        $es = $et->{content};
 
-  my $constants;
-  foreach my $constant ($dm->{constants}{constant}->@*) {
-    $self->{constants}{$constant->{name}}{type} = $constant->{type};
-    $self->{constants}{$constant->{name}}{value} = $constant->{content};
-  }
-
-  my $leg_ents;
-  foreach my $et ($dm->{entrytypes}{entrytype}->@*) {
-    my $es = $et->{content};
-
-    # Skip output flag for certain entrytypes
-    if ($et->{skip_output}) {
-      $leg_ents->{$es}{skipout} = 1;
-    }
-    # fields for entrytypes
-    my $lfs;
-    foreach my $ef ($dm->{entryfields}->@*) {
-      # Found a section describing legal fields for entrytype
-      if (not exists($ef->{entrytype}) or
-          grep {$_->{content} eq $es} $ef->{entrytype}->@*) {
-        foreach my $f ($ef->{field}->@*) {
-          $lfs->{$f->{content}} = 1;
+        # Skip output flag for certain entrytypes
+        if ($et->{skip_output}) {
+          $self->{entrytypesbyname}{$es}{skipout} = 1;
         }
       }
-    }
+      else {
+        $es = $et; # from existing config
+      }
 
-    # constraints
-    my $constraints;
-    foreach my $cd ($dm->{constraints}->@*) {
-      # Found a section describing constraints for entrytype
-      if (not exists($cd->{entrytype}) or
-          grep {$_->{content} eq $es} $cd->{entrytype}->@*) {
-        foreach my $c ($cd->{constraint}->@*) {
-          if ($c->{type} eq 'mandatory') {
-            # field
-            foreach my $f ($c->{field}->@*) {
-              push $constraints->{mandatory}->@*, $f->{content};
-            }
-            # xor set of fields
-            # [ XOR, field1, field2, ... , fieldn ]
-            foreach my $fxor ($c->{fieldxor}->@*) {
-              my $xorset;
-              foreach my $f ($fxor->{field}->@*) {
-                push $xorset->@*, $f->{content};
-              }
-              unshift $xorset->@*, 'XOR';
-              push $constraints->{mandatory}->@*, $xorset;
-            }
-            # or set of fields
-            # [ OR, field1, field2, ... , fieldn ]
-            foreach my $for ($c->{fieldor}->@*) {
-              my $orset;
-              foreach my $f ($for->{field}->@*) {
-                push $orset->@*, $f->{content};
-              }
-              unshift $orset->@*, 'OR';
-              push $constraints->{mandatory}->@*, $orset;
-            }
+      # fields for entrytypes
+      foreach my $ef ($dm->{entryfields}->@*) {
+        # Found a section describing legal fields for entrytype
+        if (not exists($ef->{entrytype}) or
+            grep {$_->{content} eq $es} $ef->{entrytype}->@*) {
+          foreach my $f ($ef->{field}->@*) {
+            $self->{entrytypesbyname}{$es}{legal_fields}{$f->{content}} = 1;
           }
-          # Conditional constraints
-          # [ ANTECEDENT_QUANTIFIER
-          #   [ ANTECEDENT LIST ]
-          #   CONSEQUENT_QUANTIFIER
-          #   [ CONSEQUENT LIST ]
-          # ]
-          elsif ($c->{type} eq 'conditional') {
-            my $cond;
-            $cond->[0] = $c->{antecedent}{quant};
-            $cond->[1] = [ map { $_->{content} } $c->{antecedent}{field}->@* ];
-            $cond->[2] = $c->{consequent}{quant};
-            $cond->[3] = [ map { $_->{content} } $c->{consequent}{field}->@* ];
-            push $constraints->{conditional}->@*, $cond;
-          }
-          # data constraints
-          elsif ($c->{type} eq 'data') {
-            my $data;
-            $data->{fields} = [ map { $_->{content} } $c->{field}->@* ];
-            $data->{datatype} = $c->{datatype};
-            $data->{rangemin} = $c->{rangemin};
-            $data->{rangemax} = $c->{rangemax};
-            $data->{pattern} = $c->{pattern};
-            push $constraints->{data}->@*, $data;
+        }
+      }
+
+      # constraints
+      foreach my $cd ($dm->{constraints}->@*) {
+        # Found a section describing constraints for entrytype
+        if (not exists($cd->{entrytype}) or
+            grep {$_->{content} eq $es} $cd->{entrytype}->@*) {
+          foreach my $c ($cd->{constraint}->@*) {
+            if ($c->{type} eq 'mandatory') {
+              # field
+              foreach my $f ($c->{field}->@*) {
+                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $f->{content};
+              }
+              # xor set of fields
+              # [ XOR, field1, field2, ... , fieldn ]
+              foreach my $fxor ($c->{fieldxor}->@*) {
+                my $xorset;
+                foreach my $f ($fxor->{field}->@*) {
+                  push $xorset->@*, $f->{content};
+                }
+                unshift $xorset->@*, 'XOR';
+                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $xorset;
+              }
+              # or set of fields
+              # [ OR, field1, field2, ... , fieldn ]
+              foreach my $for ($c->{fieldor}->@*) {
+                my $orset;
+                foreach my $f ($for->{field}->@*) {
+                  push $orset->@*, $f->{content};
+                }
+                unshift $orset->@*, 'OR';
+                push $self->{entrytypesbyname}{$es}{constraints}{mandatory}->@*, $orset;
+              }
+            }
+            # Conditional constraints
+            # [ ANTECEDENT_QUANTIFIER
+            #   [ ANTECEDENT LIST ]
+            #   CONSEQUENT_QUANTIFIER
+            #   [ CONSEQUENT LIST ]
+            # ]
+            elsif ($c->{type} eq 'conditional') {
+              my $cond;
+              $cond->[0] = $c->{antecedent}{quant};
+              $cond->[1] = [ map { $_->{content} } $c->{antecedent}{field}->@* ];
+              $cond->[2] = $c->{consequent}{quant};
+              $cond->[3] = [ map { $_->{content} } $c->{consequent}{field}->@* ];
+              push $self->{entrytypesbyname}{$es}{constraints}{conditional}->@*, $cond;
+            }
+            # data constraints
+            elsif ($c->{type} eq 'data') {
+              my $data;
+              $data->{fields} = [ map { $_->{content} } $c->{field}->@* ];
+              $data->{datatype} = $c->{datatype};
+              $data->{rangemin} = $c->{rangemin};
+              $data->{rangemax} = $c->{rangemax};
+              $data->{pattern} = $c->{pattern};
+              push $self->{entrytypesbyname}{$es}{constraints}{data}->@*, $data;
+            }
           }
         }
       }
     }
-    $leg_ents->{$es}{legal_fields} = $lfs;
-    $leg_ents->{$es}{constraints} = $constraints;
   }
-  $self->{entrytypesbyname} = $leg_ents;
-
   # Calculate and store some convenient lists of DM fields. This is to save the expense
   # of constructing these in dense loops like entry processing/output.
   # Mostly only used for .bbl output since that's the most commonly used one and so
@@ -209,8 +234,7 @@ sub new {
                       dateparts    => [sort $self->get_fields_of_type('field', 'datepart')->@*],
                       xsv       => [sort grep
                                     {
-                                      not $self->field_is_skipout($_) and
-                                        not $self->get_datatype($_) eq 'keyword'
+                                      not $self->field_is_skipout($_)
                                     }
                                     $self->get_fields_of_fieldformat('xsv')->@*],
                       ranges    => [sort grep
@@ -260,6 +284,17 @@ sub new {
   return $self;
 }
 
+=head2 get_outcase
+
+    Returns the original datamodel field/entrytype case for output
+
+=cut
+
+sub get_outcase {
+  my ($self, $string) = @_;
+  return $self->{casemap}{foldtoorig}{$string};
+}
+
 =head2 constants
 
     Returns array ref of constant names
@@ -298,6 +333,16 @@ sub get_constant_value {
   }
 }
 
+=head2 is_multiscript
+
+    Returns boolean to say if a field is a multiscript field
+
+=cut
+
+sub is_multiscript {
+  my ($self, $field) = shift;
+  return $self->{multiscriptfields}{$field} ? 1 : 0;
+}
 
 =head2 fieldtypes
 
@@ -334,8 +379,10 @@ sub is_field {
   my $self = shift;
   my $field = shift;
   my $ann = $CONFIG_META_MARKERS{annotation};
+  my $nam = $CONFIG_META_MARKERS{namedannotation};
 
-  if ($field =~ m/^(.+)(?:$ann)$/) {
+  # Ignore any annotation marker and optional annotation name
+  if ($field =~ m/^(.+)(?:$ann)(?:$nam.+)?$/) {
     return $self->{fieldsbyname}{$1} ? 1 : 0;
   }
   else {
@@ -627,12 +674,12 @@ sub check_mandatory_constraints {
   my $et = $be->get_field('entrytype');
   my $key = $be->get_field('citekey');
   my $ds = $section->get_keytods($key);
-
+# ["title", ["OR", "url", "doi", "eprint"]]
   foreach my $c ($self->{entrytypesbyname}{$et}{constraints}{mandatory}->@*) {
     if (ref($c) eq 'ARRAY') {
       # Exactly one of a set is mandatory
       if ($c->[0] eq 'XOR') {
-        my @fs = $c->@[1,-1]; # Lose the first element which is the 'XOR'
+        my @fs = $c->@[1..$#$c]; # Lose the first element which is the 'XOR'
         my $flag = 0;
         my $xorflag = 0;
         foreach my $of (@fs) {
@@ -653,7 +700,7 @@ sub check_mandatory_constraints {
       }
       # One or more of a set is mandatory
       elsif ($c->[0] eq 'OR') {
-        my @fs = $c->@[1,-1]; # Lose the first element which is the 'OR'
+        my @fs = $c->@[1..$#$c]; # Lose the first element which is the 'OR'
         my $flag = 0;
         foreach my $of (@fs) {
           if ($be->field_exists($of)) {
@@ -938,6 +985,9 @@ sub generate_bltxml_schema {
     }
   }
 
+  # Annotations
+  $writer->emptyTag('ref', 'name' => "mannotation");
+
   $writer->endTag();# interleave
   $writer->endTag();# entry element
   $writer->endTag();# oneOrMore
@@ -957,6 +1007,11 @@ sub generate_bltxml_schema {
           $writer->startTag('zeroOrMore');# for example, XDATA doesn't need a name
           $writer->startTag('element', 'name' => "$bltx:names");
 
+          $writer->startTag('choice');
+          # xdata attribute ref
+          $writer->emptyTag('ref', 'name' => 'xdata');
+
+          $writer->startTag('group');
           # useprefix attribute
           $writer->comment('useprefix option');
           $writer->startTag('optional');
@@ -989,15 +1044,16 @@ sub generate_bltxml_schema {
           $writer->emptyTag('data', 'type' => 'boolean');
           $writer->endTag();    # attribute
           $writer->endTag();    # optional
-
-          # generic annotation attribute
-          $writer->emptyTag('ref', 'name' => "annotation");
-
           $writer->startTag('oneOrMore');
 
           # Individual name element
           $writer->startTag('element', 'name' => "$bltx:name");
 
+          $writer->startTag('choice');
+          # xdata attribute ref
+          $writer->emptyTag('ref', 'name' => 'xdata');
+
+          $writer->startTag('group');
           # useprefix attribute
           $writer->comment('useprefix option');
           $writer->startTag('optional');
@@ -1016,10 +1072,6 @@ sub generate_bltxml_schema {
 
           # gender attribute ref
           $writer->emptyTag('ref', 'name' => 'gender');
-
-          # generic annotation attribute
-          $writer->emptyTag('ref', 'name' => "annotation");
-
           # namepart element
           $writer->startTag('oneOrMore');
           $writer->startTag('element', 'name' => "$bltx:namepart");
@@ -1028,17 +1080,13 @@ sub generate_bltxml_schema {
           foreach my $np ($dm->get_constant_value('nameparts')) {# list type so returns list
             $writer->dataElement('value', $np);
           }
+
           $writer->endTag();    # choice
           $writer->endTag();    # attribute
           $writer->startTag('optional');
           $writer->emptyTag('attribute', 'name' => 'initial');
           $writer->endTag();    # optional
-
-          # generic annotation attribute
-          $writer->emptyTag('ref', 'name' => "annotation");
-
           $writer->startTag('choice');
-          $writer->emptyTag('text');# text
           $writer->startTag('oneOrMore');
           $writer->startTag('element', 'name' => "$bltx:namepart");
           $writer->startTag('optional');
@@ -1047,11 +1095,16 @@ sub generate_bltxml_schema {
           $writer->emptyTag('text');# text
           $writer->endTag();    # (sub)namepart element
           $writer->endTag();    # oneOrMore
+          $writer->emptyTag('text');# text
           $writer->endTag();    # choice
           $writer->endTag();    # namepart element
           $writer->endTag();    # oneOrMore
+          $writer->endTag();    # group
+          $writer->endTag();    # choice
           $writer->endTag();    # name element
           $writer->endTag();    # oneOrMore
+          $writer->endTag();    # group
+          $writer->endTag();    # choice
           $writer->endTag();    # names element
           $writer->endTag();    # zeroOrMore
           # ========================
@@ -1063,24 +1116,22 @@ sub generate_bltxml_schema {
           foreach my $list ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$list");
-
-            # generic annotation attribute
-            $writer->emptyTag('ref', 'name' => "annotation");
-
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->startTag('choice');
             $writer->emptyTag('text');# text
             $writer->startTag('element', 'name' => "$bltx:list");
             $writer->startTag('oneOrMore');
             $writer->startTag('element', 'name' => "$bltx:item");
-
-            # generic annotation attribute
-            $writer->emptyTag('ref', 'name' => "annotation");
-
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('text');# text
+            $writer->endTag(); # choice
             $writer->endTag(); # item element
             $writer->endTag(); # oneOrMore element
             $writer->endTag(); # list element
             $writer->endTag(); # choice
+            $writer->endTag(); # choice
             $writer->endTag(); # $list element
             $writer->endTag(); # optional
           }
@@ -1094,11 +1145,10 @@ sub generate_bltxml_schema {
           foreach my $field ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
-
-            # generic annotation attribute
-            $writer->emptyTag('ref', 'name' => "annotation");
-
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('data', 'type' => 'anyURI');
+            $writer->endTag();   # choice
             $writer->endTag();   # $field element
             $writer->endTag();# optional
           }
@@ -1113,12 +1163,18 @@ sub generate_bltxml_schema {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
 
-            # generic annotation attribute
-            $writer->emptyTag('ref', 'name' => "annotation");
+            $writer->startTag('choice');
+            # xdata attribute ref
+            $writer->emptyTag('ref', 'name' => 'xdata');
 
             $writer->startTag('element', 'name' => "$bltx:list");
             $writer->startTag('oneOrMore');
             $writer->startTag('element', 'name' => "$bltx:item");
+
+            $writer->startTag('choice');
+            # xdata attribute ref
+            $writer->emptyTag('ref', 'name' => 'xdata');
+            $writer->startTag('group');
             $writer->startTag('element', 'name' => "$bltx:start");
             $writer->emptyTag('text');
             $writer->endTag();  # start element
@@ -1128,11 +1184,14 @@ sub generate_bltxml_schema {
             $writer->emptyTag('empty');
             $writer->endTag();  # choice
             $writer->endTag();  # end element
-            $writer->endTag(); # item element
-            $writer->endTag();   # oneOrMore element
+            $writer->endTag();  # group
+            $writer->endTag();  # choice
+            $writer->endTag();  # item element
+            $writer->endTag();  # oneOrMore element
             $writer->endTag();  # list element
-            $writer->endTag();   # $field element
-            $writer->endTag();# optional
+            $writer->endTag();  # choice
+            $writer->endTag();  # $field element
+            $writer->endTag();  # optional
           }
           $writer->endTag();# interleave
           # ==============================
@@ -1164,21 +1223,22 @@ sub generate_bltxml_schema {
             }
             else {
               $writer->startTag('element', 'name' => "$bltx:$field");
-
-              # generic annotation attribute
-              $writer->emptyTag('ref', 'name' => "annotation");
-
+              $writer->startTag('choice');
+              $writer->emptyTag('ref', 'name' => 'xdata');
               $writer->startTag('choice');
               $writer->startTag('list');
               $writer->startTag('oneOrMore');
               $writer->emptyTag('data', 'type' => 'string');
               $writer->endTag(); # oneOrMore
-              $writer->endTag();    # list
+              $writer->endTag(); # list
+              $writer->startTag('element', 'name' => "$bltx:list");
               $writer->startTag('oneOrMore');
-              $writer->startTag('element', 'name' => "$bltx:key");
+              $writer->startTag('element', 'name' => "$bltx:item");
               $writer->emptyTag('text');# text
-              $writer->endTag(); # key element
+              $writer->endTag(); # item element
               $writer->endTag(); # oneOrMore
+              $writer->endTag(); # list element
+              $writer->endTag(); # choice
               $writer->endTag(); # choice
               $writer->endTag(); # $field element
             }
@@ -1188,6 +1248,7 @@ sub generate_bltxml_schema {
         }
         elsif ($ft eq 'field' and $dt eq 'date') {
           # date field element definition
+          # Can't strongly type dates as we allow full ISO8601 meta characters
           # =============================
           my @types = map { s/date$//r } $dm->get_fields_of_type($ft, $dt)->@*;
           $writer->startTag('zeroOrMore');
@@ -1202,24 +1263,17 @@ sub generate_bltxml_schema {
           $writer->endTag(); # choice
           $writer->endTag(); # attribute
           $writer->endTag(); # optional
-
-          # generic annotation attribute
-          $writer->emptyTag('ref', 'name' => "annotation");
-
           $writer->startTag('choice');
-          $writer->emptyTag('data', 'type' => 'date');
-          $writer->emptyTag('data', 'type' => 'gYear');
+          $writer->emptyTag('data', 'type' => 'string');
           $writer->startTag('group');
           $writer->startTag('element', 'name' => "$bltx:start");
           $writer->startTag('choice');
-          $writer->emptyTag('data', 'type' => 'date');
-          $writer->emptyTag('data', 'type' => 'gYear');
+          $writer->emptyTag('data', 'type' => 'string');
           $writer->endTag(); # choice
           $writer->endTag(); # start element
           $writer->startTag('element', 'name' => "$bltx:end");
           $writer->startTag('choice');
-          $writer->emptyTag('data', 'type' => 'date');
-          $writer->emptyTag('data', 'type' => 'gYear');
+          $writer->emptyTag('data', 'type' => 'string');
           $writer->emptyTag('empty');
           $writer->endTag(); # choice
           $writer->endTag(); # end element
@@ -1236,11 +1290,10 @@ sub generate_bltxml_schema {
           foreach my $field ($dm->get_fields_of_type($ft, $dt)->@*) {
             $writer->startTag('optional');
             $writer->startTag('element', 'name' => "$bltx:$field");
-
-            # generic annotation attribute
-            $writer->emptyTag('ref', 'name' => "annotation");
-
+            $writer->startTag('choice');
+            $writer->emptyTag('ref', 'name' => 'xdata');
             $writer->emptyTag('text');# text
+            $writer->endTag(); # choice
             $writer->endTag(); # $field element
             $writer->endTag();# optional
           }
@@ -1252,6 +1305,18 @@ sub generate_bltxml_schema {
     }
   }
 
+  # xdata attribute definition
+  # ===========================
+  $writer->comment('xdata attribute definition');
+  $writer->startTag('define', 'name' => 'xdata');
+  $writer->startTag('optional');
+  $writer->startTag('attribute', 'name' => 'xdata');
+  $writer->emptyTag('text');# text
+  $writer->endTag();# attribute
+  $writer->endTag();# optional
+  $writer->endTag();# define
+  # ===========================
+
   # gender attribute definition
   # ===========================
   $writer->comment('gender attribute definition');
@@ -1268,13 +1333,28 @@ sub generate_bltxml_schema {
   $writer->endTag();# define
   # ===========================
 
-  # generic annotation attribute definition
-  # =====================================
-  $writer->comment('generic annotation attribute definition');
-  $writer->startTag('define', 'name' => 'annotation');
+  # generic meta annotation element definition
+  # ===========================================
+  $writer->comment('generic annotation element definition');
+  $writer->startTag('define', 'name' => 'mannotation');
+  $writer->startTag('zeroOrMore');
+  $writer->startTag('element', 'name' => "$bltx:mannotation");
+  $writer->emptyTag('attribute', 'name' => 'field');
   $writer->startTag('optional');
-  $writer->emptyTag('attribute', 'name' => 'annotation');
+  $writer->emptyTag('attribute', 'name' => 'name');
   $writer->endTag(); # optional
+  $writer->startTag('optional');
+  $writer->emptyTag('attribute', 'name' => 'item');
+  $writer->endTag(); # optional
+  $writer->startTag('optional');
+  $writer->emptyTag('attribute', 'name' => 'part');
+  $writer->endTag(); # optional
+  $writer->startTag('optional');
+  $writer->emptyTag('attribute', 'name' => 'literal');
+  $writer->endTag(); # optional
+  $writer->emptyTag('text');# text
+  $writer->endTag(); # mannotation element
+  $writer->endTag(); # zeroOrMore
   $writer->endTag();# define
   # ===========================
 
@@ -1440,6 +1520,17 @@ sub generate_bblxml_schema {
   $writer->endTag();    # inset
   $writer->endTag();    # zeroOrMore
 
+  # Per-entry options
+  $writer->startTag('zeroOrMore');
+  $writer->startTag('element', 'name' => "$bbl:options");
+  $writer->startTag('oneOrMore');
+  $writer->startTag('element', 'name' => "$bbl:option");
+  $writer->emptyTag('text');# text
+  $writer->endTag();    # option
+  $writer->endTag();    # oneOrMore
+  $writer->endTag();    # options
+  $writer->endTag();    # zeroOrMore
+
   # names
   my @names = grep {not $dm->field_is_skipout($_)} $dm->get_fields_of_type('list', 'name')->@*;
 
@@ -1456,7 +1547,7 @@ sub generate_bblxml_schema {
   $writer->emptyTag('data', 'type' => 'integer');
   $writer->endTag();    # attribute
   $writer->startTag('optional');
-  $writer->startTag('attribute', 'name' => 'uniquelist');
+  $writer->startTag('attribute', 'name' => 'ul');
   $writer->emptyTag('data', 'type' => 'integer');
   $writer->endTag();    # attribute
   $writer->endTag();    # optional
@@ -1487,7 +1578,7 @@ sub generate_bblxml_schema {
   $writer->endTag();    # optional
   $writer->emptyTag('attribute', 'name' => 'hash');
   $writer->startTag('optional');
-  $writer->startTag('attribute', 'name' => 'uniquename');
+  $writer->startTag('attribute', 'name' => 'un');
   $writer->emptyTag('data', 'type' => 'integer');
   $writer->endTag();    # attribute
   $writer->startTag('attribute', 'name' => 'uniquepart');
@@ -1499,7 +1590,7 @@ sub generate_bblxml_schema {
   $writer->emptyTag('attribute', 'name' => 'type');
   $writer->emptyTag('attribute', 'name' => 'initials');
   $writer->startTag('optional');
-  $writer->emptyTag('attribute', 'name' => 'uniquename');
+  $writer->emptyTag('attribute', 'name' => 'un');
   $writer->endTag();    # optional
   $writer->emptyTag('text');# text
   $writer->endTag();# namepart
@@ -1550,6 +1641,7 @@ sub generate_bblxml_schema {
               sortinit
               sortinithash
               sortinithash
+              extraname
               extradate
               labelyear
               labelmonth
@@ -1741,6 +1833,13 @@ sub generate_bblxml_schema {
     $writer->endTag();          # optional
   }
 
+  # nocite
+  $writer->startTag('optional');
+  $writer->startTag('element', 'name' => "$bbl:nocite");
+  $writer->emptyTag('empty');
+  $writer->endTag();# nocite
+  $writer->endTag();# optional
+
   # keywords
   $writer->startTag('optional');
   $writer->startTag('element', 'name' => "$bbl:keywords");
@@ -1763,7 +1862,15 @@ sub generate_bblxml_schema {
   $writer->endTag();# choice
   $writer->endTag();# scope attribute
   $writer->emptyTag('attribute', 'name' => 'field');
+  $writer->emptyTag('attribute', 'name' => 'name');
   $writer->emptyTag('attribute', 'name' => 'value');
+  $writer->startTag('attribute', 'name' => 'literal');
+  $writer->startTag('choice');
+  foreach my $s ('1', '0') {
+    $writer->dataElement('value', $s);
+  }
+  $writer->endTag();# choice
+  $writer->endTag();# literal attribute
   $writer->startTag('optional');
   $writer->emptyTag('attribute', 'name' => 'item');
   $writer->endTag();# optional
@@ -1819,7 +1926,6 @@ __END__
 
 =head1 AUTHORS
 
-François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -1829,7 +1935,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2018 François Charette and Philip Kime, all rights reserved.
+Copyright 2012-2019 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
