@@ -1,5 +1,5 @@
 package Biber::Sections;
-use v5.16;
+use v5.24;
 use strict;
 use warnings;
 
@@ -29,8 +29,8 @@ sub new {
 
 sub get_num_sections {
   my $self = shift;
-  my @keys = keys %$self;
- return $#keys + 1;
+  my @keys = keys $self->%*;
+  return $#keys + 1;
 }
 
 
@@ -54,7 +54,7 @@ sub get_section {
 
 sub get_sections {
   my $self = shift;
-  return [ sort {$a->number <=> $b->number} values %$self ];
+  return [ sort {$a->number <=> $b->number} values $self->%* ];
 }
 
 
@@ -93,7 +93,6 @@ __END__
 
 =head1 AUTHORS
 
-François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -103,7 +102,8 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2014 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
+Copyright 2012-2019 Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
